﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

// At design time:
//   Set the ImageList's ImageSize properties to the correct values:
//      imlSmallIcons.ImageSize = 32,32
//      imlLargeIcons.ImageSize = 64,64
//   Set the ImageList's ColorDepth properties to the correct values:
//      imlSmallIcons.ColorDepth = Depth32bit
//      imlLargeIcons.ColorDepth = Depth32bit

using System.Drawing.Drawing2D;

namespace howto_listview_custom_icons
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            // Select the first style.
            cboStyle.SelectedIndex = 0;

            // Initialize the ListView.
            lvwBooks.SmallImageList = imlSmallIcons;
            lvwBooks.LargeImageList = imlLargeIcons;

            // Make the column headers.
            lvwBooks.MakeColumnHeaders(
                "Title", 230, HorizontalAlignment.Left,
                "URL", 220, HorizontalAlignment.Left,
                "ISBN", 130, HorizontalAlignment.Left,
                "Picture", 230, HorizontalAlignment.Left,
                "Pages", 50, HorizontalAlignment.Right,
                "Year", 60, HorizontalAlignment.Right);

            // Create images.
            RectangleF small_rect = new RectangleF(0, 0, 32, 32);
            RectangleF large_rect = new RectangleF(3, 3, 58, 58);
            imlSmallIcons.Images.Clear();
            imlLargeIcons.Images.Clear();
            using (Pen small_pen = new Pen(Color.Blue, 2))
            {
                using (Pen large_pen = new Pen(Color.Blue, 3))
                {
                    large_pen.LineJoin = LineJoin.Round;
                    for (int i = 1; i <= 7; i++)
                    {
                        Bitmap bm32x32 = new Bitmap(32, 32);
                        using (Graphics gr = Graphics.FromImage(bm32x32))
                        {
                            gr.SmoothingMode = SmoothingMode.AntiAlias;
                            gr.Clear(Color.Transparent);
                            DrawStar(gr, small_rect, i + 3, small_pen, Brushes.Yellow);
                            // Save the image using i as the image's key.
                            imlSmallIcons.Images.Add(i.ToString(), bm32x32);
                        }

                        Bitmap bm64x64 = new Bitmap(64, 64);
                        using (Graphics gr = Graphics.FromImage(bm64x64))
                        {
                            gr.Clear(Color.Transparent);
                            gr.SmoothingMode = SmoothingMode.AntiAlias;
                            DrawStar(gr, large_rect, i + 3, large_pen, Brushes.Yellow);
                            // Save the image using i as the image's key.
                            imlLargeIcons.Images.Add(i.ToString(), bm64x64);
                        }
                    }
                }
            }

            // Add data rows.
            lvwBooks.AddRow("1", "WPF 3d", "http://www.csharphelper.com/wpf3d.html", "978-1983905964", "http://www.csharphelper.com/wpf3d_350_429.jpg", "430", "2018");
            lvwBooks.AddRow("2", "The C# Helper Top 100", "http://www.csharphelper.com/top100.htm", "978-1546886716", "http://www.csharphelper.com/top100_350_433.jpg", "380", "2017");
            lvwBooks.AddRow("3", "Interview Puzzles Dissected", "http://www.csharphelper.com/puzzles.htm", "978-1539504887", "http://www.csharphelper.com/interview_puzzles_350_433.jpg", "300", "2016");
            lvwBooks.AddRow("4", "C# 24-Hour Trainer, 2e", "http://tinyurl.com/n2a5797", "978-1-119-06566-1", "http://www.csharphelper.com/csharp24hr_2e_79x100.jpg", "600", "2015");
            lvwBooks.AddRow("5", "Beginning Software Engineering", "http://tinyurl.com/pz7bavo", "978-1-118-96914-4", "http://tinyurl.com/y7zusrct", "480", "2015");
            lvwBooks.AddRow("6", "Essential Algorithms", "http://tinyurl.com/y9uqajqv", "978-1-118-61210-1", "http://tinyurl.com/y84d2jgp", "624", "2013");
            lvwBooks.AddRow("7", "Beginning Database Design Solutions", "http://www.vb-helper.com/db_design.htm", "978-0-470-38549-4", "http://www.vb-helper.com/db_design.jpg", "522", "2008");
        }

        // Change the ListView's display style.
        private void cboStyle_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cboStyle.Text)
            {
                case "Large Icons":
                    lvwBooks.View = View.LargeIcon;
                    break;
                case "Small Icons":
                    lvwBooks.View = View.SmallIcon;
                    break;
                case "List":
                    lvwBooks.View = View.List;
                    break;
                case "Tile":
                    lvwBooks.View = View.Tile;
                    break;
                case "Details":
                    lvwBooks.View = View.Details;
                    break;
            }
        }

        // Draw a star with the indicated number of points.
        private void DrawStar(Graphics gr, RectangleF rect, int num_points, Pen pen, Brush brush)
        {
            // Compute geometry.
            float cx = (rect.Left + rect.Right) / 2f;
            float cy = (rect.Top + rect.Bottom) / 2f;
            float rx1 = rect.Width / 2f - 1;
            float ry1 = rect.Height / 2f - 1;
            float rx2 = rx1 / 3f;
            float ry2 = ry1 / 3f;

            // Generate the star's points.
            double theta = -Math.PI / 2;
            double dtheta = Math.PI / num_points;
            List<PointF> points = new List<PointF>();
            for (int i = 0; i < num_points; i++)
            {
                points.Add(new PointF(
                    (float)(cx + rx1 * Math.Cos(theta)),
                    (float)(cy + ry1 * Math.Sin(theta))));
                theta += dtheta;
                points.Add(new PointF(
                    (float)(cx + rx2 * Math.Cos(theta)),
                    (float)(cy + ry2 * Math.Sin(theta))));
                theta += dtheta;
            }

            // Draw.
            PointF[] point_array = points.ToArray();
            gr.FillPolygon(brush, point_array);
            gr.DrawPolygon(pen, point_array);
        }
    }
}
